@@ -10,7 +10,7 @@ from typing import List, Dict, Any, Optional, TYPE_CHECKING
 from parted import (  # type: ignore
 	Disk, Geometry, FileSystem,
 	PartitionException, DiskLabelException,
-	getAllDevices, freshDisk, Partition, Device
+	getDevice, getAllDevices, freshDisk, Partition, Device
 )
 
 from .device_model import (
@@ -45,7 +45,17 @@ class DeviceHandler(object):
 	def load_devices(self):
 		block_devices = {}
 
-		for device in getAllDevices():
+		devices = getAllDevices()
+
+		try:
+			loop_devices = SysCommand(['losetup', '-a'])
+			for ld_info in str(loop_devices).splitlines():
+				loop_device = getDevice(ld_info.split(':', maxsplit=1)[0])
+				devices.append(loop_device)
+		except Exception as err:
+			debug(f'Failed to get loop devices: {err}')
+
+		for device in devices:
 			if get_lsblk_info(device.path).type == 'rom':
 				continue
 
@@ -117,6 +127,10 @@ class DeviceHandler(object):
 				return part
 		return None
 
+	def get_parent_device_path(self, dev_path: Path) -> Path:
+		lsblk = get_lsblk_info(dev_path)
+		return Path(f'/dev/{lsblk.pkname}')
+
 	def get_uuid_for_path(self, path: Path) -> Optional[str]:
 		partition = self.find_partition(path)
 		return partition.partuuid if partition else None
@@ -242,7 +256,7 @@ class DeviceHandler(object):
 		info(f'luks2 locking device: {dev_path}')
 		luks_handler.lock()
 
-	def _validate(self, device_mod: DeviceModification):
+	def _validate_partitions(self, partitions: List[PartitionModification]):
 		checks = {
 			# verify that all partitions have a path set (which implies that they have been created)
 			lambda x: x.dev_path is None: ValueError('When formatting, all partitions must have a path set'),
@@ -253,7 +267,7 @@ class DeviceHandler(object):
 		}
 
 		for check, exc in checks.items():
-			found = next(filter(check, device_mod.partitions), None)
+			found = next(filter(check, partitions), None)
 			if found is not None:
 				raise exc
 
@@ -266,12 +280,16 @@ class DeviceHandler(object):
 		Format can be given an overriding path, for instance /dev/null to test
 		the formatting functionality and in essence the support for the given filesystem.
 		"""
-		self._validate(device_mod)
+
+		# don't touch existing partitions
+		filtered_part = [p for p in device_mod.partitions if not p.exists()]
+
+		self._validate_partitions(filtered_part)
 
 		# make sure all devices are unmounted
-		self._umount_all_existing(device_mod)
+		self._umount_all_existing(device_mod.device_path)
 
-		for part_mod in device_mod.partitions:
+		for part_mod in filtered_part:
 			# partition will be encrypted
 			if enc_conf is not None and part_mod in enc_conf.partitions:
 				self._perform_enc_formatting(
@@ -283,6 +301,12 @@ class DeviceHandler(object):
 			else:
 				self._perform_formatting(part_mod.safe_fs_type, part_mod.safe_dev_path)
 
+			lsblk_info = self._fetch_part_info(part_mod.safe_dev_path)
+
+			part_mod.partn = lsblk_info.partn
+			part_mod.partuuid = lsblk_info.partuuid
+			part_mod.uuid = lsblk_info.uuid
+
 	def _perform_partitioning(
 		self,
 		part_mod: PartitionModification,
@@ -346,15 +370,10 @@ class DeviceHandler(object):
 
 			# the partition has a real path now as it was created
 			part_mod.dev_path = Path(partition.path)
-
-			lsblk_info = self._fetch_partuuid(part_mod.dev_path)
-
-			part_mod.partuuid = lsblk_info.partuuid
-			part_mod.uuid = lsblk_info.uuid
 		except PartitionException as ex:
 			raise DiskError(f'Unable to add partition, most likely due to overlapping sectors: {ex}') from ex
 
-	def _fetch_partuuid(self, path: Path) -> LsblkInfo:
+	def _fetch_part_info(self, path: Path) -> LsblkInfo:
 		attempts = 3
 		lsblk_info: Optional[LsblkInfo] = None
 
@@ -363,16 +382,28 @@ class DeviceHandler(object):
 			time.sleep(attempt_nr + 1)
 			lsblk_info = get_lsblk_info(path)
 
-			if lsblk_info.partuuid:
+			if lsblk_info.partn and lsblk_info.partuuid and lsblk_info.uuid:
 				break
 
 			self.partprobe(path)
 
-		if not lsblk_info or not lsblk_info.partuuid:
+		if not lsblk_info:
+			debug(f'Unable to get partition information: {path}')
+			raise DiskError(f'Unable to get partition information: {path}')
+
+		if not lsblk_info.partn:
+			debug(f'Unable to determine new partition number: {path}\n{lsblk_info}')
+			raise DiskError(f'Unable to determine new partition number: {path}')
+
+		if not lsblk_info.partuuid:
 			debug(f'Unable to determine new partition uuid: {path}\n{lsblk_info}')
 			raise DiskError(f'Unable to determine new partition uuid: {path}')
 
-		debug(f'partuuid found: {lsblk_info.json()}')
+		if not lsblk_info.uuid:
+			debug(f'Unable to determine new uuid: {path}\n{lsblk_info}')
+			raise DiskError(f'Unable to determine new uuid: {path}')
+
+		debug(f'partition information found: {lsblk_info.json()}')
 
 		return lsblk_info
 
@@ -442,10 +473,10 @@ class DeviceHandler(object):
 
 		return luks_handler
 
-	def _umount_all_existing(self, modification: DeviceModification):
-		info(f'Unmounting all partitions: {modification.device_path}')
+	def _umount_all_existing(self, device_path: Path):
+		info(f'Unmounting all existing partitions: {device_path}')
 
-		existing_partitions = self._devices[modification.device_path].partition_infos
+		existing_partitions = self._devices[device_path].partition_infos
 
 		for partition in existing_partitions:
 			debug(f'Unmounting: {partition.path}')
@@ -472,7 +503,7 @@ class DeviceHandler(object):
 				raise DiskError('Too many partitions on disk, MBR disks can only have 3 primary partitions')
 
 		# make sure all devices are unmounted
-		self._umount_all_existing(modification)
+		self._umount_all_existing(modification.device_path)
 
 		# WARNING: the entire device will be wiped and all data lost
 		if modification.wipe:
@@ -485,13 +516,10 @@ class DeviceHandler(object):
 
 		info(f'Creating partitions: {modification.device_path}')
 
-		# TODO sort by delete first
-
-		for part_mod in modification.partitions:
-			# don't touch existing partitions
-			if part_mod.exists():
-				continue
+		# don't touch existing partitions
+		filtered_part = [p for p in modification.partitions if not p.exists()]
 
+		for part_mod in filtered_part:
 			# if the entire disk got nuked then we don't have to delete
 			# any existing partitions anymore because they're all gone already
 			requires_delete = modification.wipe is False
@@ -518,14 +546,18 @@ class DeviceHandler(object):
 			info(f'Device already mounted at {target_mountpoint}')
 			return
 
-		str_options = ','.join(options)
-		str_options = f'-o {str_options}' if str_options else ''
+		cmd = ['mount']
 
-		mount_fs = f'-t {mount_fs}' if mount_fs else ''
+		if len(options):
+			cmd.extend(('-o', ','.join(options)))
+		if mount_fs:
+			cmd.extend(('-t', mount_fs))
 
-		command = f'mount {mount_fs} {str_options} {dev_path} {target_mountpoint}'
+		cmd.extend((str(dev_path), str(target_mountpoint)))
 
-		debug(f'Mounting {dev_path}: command')
+		command = ' '.join(cmd)
+
+		debug(f'Mounting {dev_path}: {command}')
 
 		try:
 			SysCommand(command)
@@ -565,7 +597,9 @@ class DeviceHandler(object):
 					if is_subpath(mountpoint, base_mountpoint):
 						path = Path(part_info.disk.device.path)
 						part_mods.setdefault(path, [])
-						part_mods[path].append(PartitionModification.from_existing_partition(part_info))
+						part_mod = PartitionModification.from_existing_partition(part_info)
+						part_mod.mountpoint = mountpoint.root / mountpoint.relative_to(base_mountpoint)
+						part_mods[path].append(part_mod)
 						break
 
 		device_mods: List[DeviceModification] = []
