@@ -20,6 +20,7 @@ from ..exceptions import DiskError, SysCallError
 from ..general import SysCommand
 from ..output import debug, error
 from ..storage import storage
+from ..output import info
 
 if TYPE_CHECKING:
 	_: Any
@@ -41,17 +42,11 @@ class DiskLayoutType(Enum):
 class DiskLayoutConfiguration:
 	config_type: DiskLayoutType
 	device_modifications: List[DeviceModification] = field(default_factory=list)
-	# used for pre-mounted config
-	relative_mountpoint: Optional[Path] = None
 
-	def __post_init__(self):
-		if self.config_type == DiskLayoutType.Pre_mount and self.relative_mountpoint is None:
-			raise ValueError('Must set a relative mountpoint when layout type is pre-mount"')
-
-	def __dump__(self) -> Dict[str, Any]:
+	def json(self) -> Dict[str, Any]:
 		return {
 			'config_type': self.config_type.value,
-			'device_modifications': [mod.__dump__() for mod in self.device_modifications]
+			'device_modifications': [mod.json() for mod in self.device_modifications]
 		}
 
 	@classmethod
@@ -92,9 +87,10 @@ class DiskLayoutConfiguration:
 					status=ModificationStatus(partition['status']),
 					fs_type=FilesystemType(partition['fs_type']),
 					start=Size.parse_args(partition['start']),
-					length=Size.parse_args(partition['length']),
+					length=Size.parse_args(partition['size']),
 					mount_options=partition['mount_options'],
 					mountpoint=Path(partition['mountpoint']) if partition['mountpoint'] else None,
+					dev_path=Path(partition['dev_path']) if partition['dev_path'] else None,
 					type=PartitionType(partition['type']),
 					flags=[PartitionFlag[f] for f in partition.get('flags', [])],
 					btrfs_subvols=SubvolumeModification.parse_args(partition.get('btrfs', [])),
@@ -136,80 +132,89 @@ class Unit(Enum):
 
 	sectors = 'sectors'  # size in sector
 
-	Percent = '%' 	# size in percentile
-
 	@staticmethod
 	def get_all_units() -> List[str]:
 		return [u.name for u in Unit]
 
+	@staticmethod
+	def get_si_units() -> List[Unit]:
+		return [u for u in Unit if 'i' not in u.name and u.name != 'sectors']
+
+
+@dataclass
+class SectorSize:
+	value: int
+	unit: Unit
+
+	def __post_init__(self):
+		match self.unit:
+			case Unit.sectors:
+				raise ValueError('Unit type sector not allowed for SectorSize')
+
+	@staticmethod
+	def default() -> SectorSize:
+		return SectorSize(512, Unit.B)
+
+	def json(self) -> Dict[str, Any]:
+		return {
+			'value': self.value,
+			'unit': self.unit.name,
+		}
+
+	@classmethod
+	def parse_args(cls, arg: Dict[str, Any]) -> SectorSize:
+		return SectorSize(
+			arg['value'],
+			Unit[arg['unit']]
+		)
+
+	def normalize(self) -> int:
+		"""
+		will normalize the value of the unit to Byte
+		"""
+		return int(self.value * self.unit.value)  # type: ignore
+
 
 @dataclass
 class Size:
 	value: int
 	unit: Unit
-	sector_size: Optional[Size] = None  # only required when unit is sector
-	total_size: Optional[Size] = None  # required when operating on percentages
+	sector_size: SectorSize
 
 	def __post_init__(self):
-		if self.unit == Unit.sectors and self.sector_size is None:
-			raise ValueError('Sector size is required when unit is sectors')
-		elif self.unit == Unit.Percent:
-			if self.value < 0 or self.value > 100:
-				raise ValueError('Percentage must be between 0 and 100')
-			elif self.total_size is None:
-				raise ValueError('Total size is required when unit is percentage')
+		if not isinstance(self.sector_size, SectorSize):
+			raise ValueError('sector size must be of type SectorSize')
 
-	@property
-	def _total_size(self) -> Size:
-		"""
-		Save method to get the total size, mainly to satisfy mypy
-		This shouldn't happen as the Size object fails instantiation on missing total size
-		"""
-		if self.unit == Unit.Percent and self.total_size is None:
-			raise ValueError('Percent unit size must specify a total size')
-		return self.total_size  # type: ignore
-
-	def __dump__(self) -> Dict[str, Any]:
+	def json(self) -> Dict[str, Any]:
 		return {
 			'value': self.value,
 			'unit': self.unit.name,
-			'sector_size': self.sector_size.__dump__() if self.sector_size else None,
-			'total_size': self._total_size.__dump__() if self._total_size else None
+			'sector_size': self.sector_size.json() if self.sector_size else None
 		}
 
 	@classmethod
 	def parse_args(cls, size_arg: Dict[str, Any]) -> Size:
 		sector_size = size_arg['sector_size']
-		total_size = size_arg['total_size']
 
 		return Size(
 			size_arg['value'],
 			Unit[size_arg['unit']],
-			Size.parse_args(sector_size) if sector_size else None,
-			Size.parse_args(total_size) if total_size else None
+			SectorSize.parse_args(sector_size),
 		)
 
 	def convert(
 		self,
 		target_unit: Unit,
-		sector_size: Optional[Size] = None,
-		total_size: Optional[Size] = None
+		sector_size: Optional[SectorSize] = None
 	) -> Size:
 		if target_unit == Unit.sectors and sector_size is None:
 			raise ValueError('If target has unit sector, a sector size must be provided')
 
-		# not sure why we would ever wanna convert to percentages
-		if target_unit == Unit.Percent and total_size is None:
-			raise ValueError('Missing paramter total size to be able to convert to percentage')
-
 		if self.unit == target_unit:
 			return self
-		elif self.unit == Unit.Percent:
-			amount = int(self._total_size._normalize() * (self.value / 100))
-			return Size(amount, Unit.B)
 		elif self.unit == Unit.sectors:
 			norm = self._normalize()
-			return Size(norm, Unit.B).convert(target_unit, sector_size)
+			return Size(norm, Unit.B, self.sector_size).convert(target_unit, sector_size)
 		else:
 			if target_unit == Unit.sectors and sector_size is not None:
 				norm = self._normalize()
@@ -217,7 +222,7 @@ class Size:
 				return Size(sectors, Unit.sectors, sector_size)
 			else:
 				value = int(self._normalize() / target_unit.value)  # type: ignore
-				return Size(value, target_unit)
+				return Size(value, target_unit, self.sector_size)
 
 	def as_text(self) -> str:
 		return self.format_size(
@@ -228,31 +233,45 @@ class Size:
 	def format_size(
 		self,
 		target_unit: Unit,
-		sector_size: Optional[Size] = None,
+		sector_size: Optional[SectorSize] = None,
 		include_unit: bool = True
 	) -> str:
-		if self.unit == Unit.Percent:
-			return f'{self.value}%'
-		else:
-			target_size = self.convert(target_unit, sector_size)
-			if include_unit:
-				return f'{target_size.value} {target_unit.name}'
-			return f'{target_size.value}'
+		target_size = self.convert(target_unit, sector_size)
+
+		if include_unit:
+			return f'{target_size.value} {target_unit.name}'
+		return f'{target_size.value}'
+
+	def format_highest(self, include_unit: bool = True) -> str:
+		si_units = Unit.get_si_units()
+		all_si_values = [self.convert(si) for si in si_units]
+		filtered = filter(lambda x: x.value >= 1, all_si_values)
+
+		# we have to get the max by the unit value as we're interested
+		# in getting the value in the highest possible unit without floats
+		si_value = max(filtered, key=lambda x: x.unit.value)
+
+		if include_unit:
+			return f'{si_value.value} {si_value.unit.name}'
+		return f'{si_value.value}'
 
 	def _normalize(self) -> int:
 		"""
 		will normalize the value of the unit to Byte
 		"""
-		if self.unit == Unit.Percent:
-			return self.convert(Unit.B).value
-		elif self.unit == Unit.sectors and self.sector_size is not None:
-			return self.value * self.sector_size._normalize()
+		if self.unit == Unit.sectors and self.sector_size is not None:
+			return self.value * self.sector_size.normalize()
 		return int(self.value * self.unit.value)  # type: ignore
 
 	def __sub__(self, other: Size) -> Size:
 		src_norm = self._normalize()
 		dest_norm = other._normalize()
-		return Size(abs(src_norm - dest_norm), Unit.B)
+		return Size(abs(src_norm - dest_norm), Unit.B, self.sector_size)
+
+	def __add__(self, other: Size) -> Size:
+		src_norm = self._normalize()
+		dest_norm = other._normalize()
+		return Size(abs(src_norm + dest_norm), Unit.B, self.sector_size)
 
 	def __lt__(self, other):
 		return self._normalize() < other._normalize()
@@ -294,14 +313,22 @@ class _PartitionInfo:
 	mountpoints: List[Path]
 	btrfs_subvol_infos: List[_BtrfsSubvolumeInfo] = field(default_factory=list)
 
+	@property
+	def sector_size(self) -> SectorSize:
+		sector_size = self.partition.geometry.device.sectorSize
+		return SectorSize(sector_size, Unit.B)
+
 	def table_data(self) -> Dict[str, Any]:
+		end = self.start + self.length
+
 		part_info = {
 			'Name': self.name,
 			'Type': self.type.value,
 			'Filesystem': self.fs_type.value if self.fs_type else str(_('Unknown')),
 			'Path': str(self.path),
-			'Start': self.start.format_size(Unit.MiB),
-			'Length': self.length.format_size(Unit.MiB),
+			'Start': self.start.format_size(Unit.sectors, self.sector_size, include_unit=False),
+			'End': end.format_size(Unit.sectors, self.sector_size, include_unit=False),
+			'Size': self.length.format_highest(),
 			'Flags': ', '.join([f.name for f in self.flags])
 		}
 
@@ -325,10 +352,14 @@ class _PartitionInfo:
 		start = Size(
 			partition.geometry.start,
 			Unit.sectors,
-			Size(partition.disk.device.sectorSize, Unit.B)
+			SectorSize(partition.disk.device.sectorSize, Unit.B)
 		)
 
-		length = Size(int(partition.getLength(unit='B')), Unit.B)
+		length = Size(
+			int(partition.getLength(unit='B')),
+			Unit.B,
+			SectorSize(partition.disk.device.sectorSize, Unit.B)
+		)
 
 		return _PartitionInfo(
 			partition=partition,
@@ -353,7 +384,7 @@ class _DeviceInfo:
 	type: str
 	total_size: Size
 	free_space_regions: List[DeviceGeometry]
-	sector_size: Size
+	sector_size: SectorSize
 	read_only: bool
 	dirty: bool
 
@@ -363,7 +394,7 @@ class _DeviceInfo:
 			'Model': self.model,
 			'Path': str(self.path),
 			'Type': self.type,
-			'Size': self.total_size.format_size(Unit.MiB),
+			'Size': self.total_size.format_highest(),
 			'Free space': int(total_free_space),
 			'Sector size': self.sector_size.value,
 			'Read only': self.read_only
@@ -372,17 +403,22 @@ class _DeviceInfo:
 	@classmethod
 	def from_disk(cls, disk: Disk) -> _DeviceInfo:
 		device = disk.device
-		device_type = parted.devices[device.type]
+		if device.type == 18:
+			device_type = 'loop'
+		else:
+			device_type = parted.devices[device.type]
 
-		sector_size = Size(device.sectorSize, Unit.B)
+		sector_size = SectorSize(device.sectorSize, Unit.B)
 		free_space = [DeviceGeometry(g, sector_size) for g in disk.getFreeSpaceRegions()]
 
+		sector_size = SectorSize(device.sectorSize, Unit.B)
+
 		return _DeviceInfo(
 			model=device.model.strip(),
 			path=Path(device.path),
 			type=device_type,
 			sector_size=sector_size,
-			total_size=Size(int(device.getLength(unit='B')), Unit.B),
+			total_size=Size(int(device.getLength(unit='B')), Unit.B, sector_size),
 			free_space_regions=free_space,
 			read_only=device.readOnly,
 			dirty=device.dirty
@@ -410,12 +446,18 @@ class SubvolumeModification:
 
 			mountpoint = Path(entry['mountpoint']) if entry['mountpoint'] else None
 
+			compress = entry.get('compress', False)
+			nodatacow = entry.get('nodatacow', False)
+
+			if compress and nodatacow:
+				raise ValueError('compress and nodatacow flags cannot be enabled simultaneously on a btfrs subvolume')
+
 			mods.append(
 				SubvolumeModification(
 					entry['name'],
 					mountpoint,
-					entry.get('compress', False),
-					entry.get('nodatacow', False)
+					compress,
+					nodatacow
 				)
 			)
 
@@ -439,14 +481,12 @@ class SubvolumeModification:
 
 		raise ValueError('Mountpoint is not specified')
 
-	def is_root(self, relative_mountpoint: Optional[Path] = None) -> bool:
+	def is_root(self) -> bool:
 		if self.mountpoint:
-			if relative_mountpoint is not None:
-				return self.mountpoint.relative_to(relative_mountpoint) == Path('.')
 			return self.mountpoint == Path('/')
 		return False
 
-	def __dump__(self) -> Dict[str, Any]:
+	def json(self) -> Dict[str, Any]:
 		return {
 			'name': str(self.name),
 			'mountpoint': str(self.mountpoint),
@@ -455,16 +495,11 @@ class SubvolumeModification:
 		}
 
 	def table_data(self) -> Dict[str, Any]:
-		return {
-			'name': str(self.name),
-			'mountpoint': str(self.mountpoint),
-			'compress': self.compress,
-			'nodatacow': self.nodatacow
-		}
+		return self.json()
 
 
 class DeviceGeometry:
-	def __init__(self, geometry: Geometry, sector_size: Size):
+	def __init__(self, geometry: Geometry, sector_size: SectorSize):
 		self._geometry = geometry
 		self._sector_size = sector_size
 
@@ -492,7 +527,7 @@ class DeviceGeometry:
 			'Sector size': self._sector_size.value,
 			'Start (sector/B)': start_str,
 			'End (sector/B)': end_str,
-			'Length (sectors/B)': length_str
+			'Size (sectors/B)': length_str
 		}
 
 
@@ -509,13 +544,15 @@ class BDevice:
 class PartitionType(Enum):
 	Boot = 'boot'
 	Primary = 'primary'
+	_Unknown = 'unknown'
 
 	@classmethod
 	def get_type_from_code(cls, code: int) -> PartitionType:
 		if code == parted.PARTITION_NORMAL:
 			return PartitionType.Primary
-
-		raise DiskError(f'Partition code not supported: {code}')
+		else:
+			info(f'Partition code not supported: {code}')
+			return PartitionType._Unknown
 
 	def get_partition_code(self) -> Optional[int]:
 		if self == PartitionType.Primary:
@@ -617,10 +654,12 @@ class PartitionModification:
 
 	# only set if the device was created or exists
 	dev_path: Optional[Path] = None
+	partn: Optional[int] = None
 	partuuid: Optional[str] = None
 	uuid: Optional[str] = None
 
-	_boot_indicator_flags = [PartitionFlag.Boot, PartitionFlag.XBOOTLDR]
+	_efi_indicator_flags = (PartitionFlag.Boot, PartitionFlag.ESP)
+	_boot_indicator_flags = (PartitionFlag.Boot, PartitionFlag.XBOOTLDR)
 
 	def __post_init__(self):
 		# needed to use the object as a dictionary key due to hash func
@@ -659,9 +698,9 @@ class PartitionModification:
 		if partition_info.btrfs_subvol_infos:
 			mountpoint = None
 			subvol_mods = []
-			for info in partition_info.btrfs_subvol_infos:
+			for i in partition_info.btrfs_subvol_infos:
 				subvol_mods.append(
-					SubvolumeModification.from_existing_subvol_info(info)
+					SubvolumeModification.from_existing_subvol_info(i)
 				)
 		else:
 			mountpoint = partition_info.mountpoints[0] if partition_info.mountpoints else None
@@ -690,20 +729,25 @@ class PartitionModification:
 
 		raise ValueError('Mountpoint is not specified')
 
+	def is_efi(self) -> bool:
+		return (
+			any(set(self.flags) & set(self._efi_indicator_flags))
+			and self.fs_type == FilesystemType.Fat32
+			and PartitionFlag.XBOOTLDR not in self.flags
+		)
+
 	def is_boot(self) -> bool:
 		"""
 		Returns True if any of the boot indicator flags are found in self.flags
 		"""
 		return any(set(self.flags) & set(self._boot_indicator_flags))
 
-	def is_root(self, relative_mountpoint: Optional[Path] = None) -> bool:
-		if relative_mountpoint is not None and self.mountpoint is not None:
-			return self.mountpoint.relative_to(relative_mountpoint) == Path('.')
-		elif self.mountpoint is not None:
+	def is_root(self) -> bool:
+		if self.mountpoint is not None:
 			return Path('/') == self.mountpoint
 		else:
 			for subvol in self.btrfs_subvols:
-				if subvol.is_root(relative_mountpoint):
+				if subvol.is_root():
 					return True
 
 		return False
@@ -741,25 +785,29 @@ class PartitionModification:
 			'obj_id': self.obj_id,
 			'status': self.status.value,
 			'type': self.type.value,
-			'start': self.start.__dump__(),
-			'length': self.length.__dump__(),
+			'start': self.start.json(),
+			'size': self.length.json(),
 			'fs_type': self.fs_type.value if self.fs_type else '',
 			'mountpoint': str(self.mountpoint) if self.mountpoint else None,
 			'mount_options': self.mount_options,
 			'flags': [f.name for f in self.flags],
-			'btrfs': [vol.__dump__() for vol in self.btrfs_subvols]
+			'dev_path': str(self.dev_path) if self.dev_path else None,
+			'btrfs': [vol.json() for vol in self.btrfs_subvols]
 		}
 
 	def table_data(self) -> Dict[str, Any]:
 		"""
 		Called for displaying data in table format
 		"""
+		end = self.start + self.length
+
 		part_mod = {
 			'Status': self.status.value,
 			'Device': str(self.dev_path) if self.dev_path else '',
 			'Type': self.type.value,
-			'Start': self.start.format_size(Unit.MiB),
-			'Length': self.length.format_size(Unit.MiB),
+			'Start': self.start.format_size(Unit.sectors, self.start.sector_size, include_unit=False),
+			'End': end.format_size(Unit.sectors, self.start.sector_size, include_unit=False),
+			'Size': self.length.format_highest(),
 			'FS type': self.fs_type.value if self.fs_type else 'Unknown',
 			'Mountpoint': self.mountpoint if self.mountpoint else '',
 			'Mount options': ', '.join(self.mount_options),
@@ -789,13 +837,14 @@ class DeviceModification:
 		"""
 		Similar to get_boot_partition() but excludes XBOOTLDR partitions from it's candidates.
 		"""
-		filtered = filter(lambda x: x.is_boot() and x.fs_type == FilesystemType.Fat32 and PartitionFlag.XBOOTLDR not in x.flags, self.partitions)
+		filtered = filter(lambda x: x.is_efi() and x.mountpoint, self.partitions)
 		return next(filtered, None)
 
 	def get_boot_partition(self) -> Optional[PartitionModification]:
 		"""
 		Returns the first partition marked as XBOOTLDR (PARTTYPE id of bc13c2ff-...) or Boot and has a mountpoint.
 		Only returns XBOOTLDR if separate EFI is detected using self.get_efi_partition()
+		Will return None if no suitable partition is found.
 		"""
 		if efi_partition := self.get_efi_partition():
 			filtered = filter(lambda x: x.is_boot() and x != efi_partition and x.mountpoint, self.partitions)
@@ -806,11 +855,11 @@ class DeviceModification:
 			filtered = filter(lambda x: x.is_boot() and x.mountpoint, self.partitions)
 			return next(filtered, None)
 
-	def get_root_partition(self, relative_path: Optional[Path]) -> Optional[PartitionModification]:
-		filtered = filter(lambda x: x.is_root(relative_path), self.partitions)
+	def get_root_partition(self) -> Optional[PartitionModification]:
+		filtered = filter(lambda x: x.is_root(), self.partitions)
 		return next(filtered, None)
 
-	def __dump__(self) -> Dict[str, Any]:
+	def json(self) -> Dict[str, Any]:
 		"""
 		Called when generating configuration files
 		"""
@@ -902,6 +951,13 @@ class Fido2Device:
 			'product': self.product
 		}
 
+	def table_data(self) -> Dict[str, str]:
+		return {
+			'Path': str(self.path),
+			'Manufacturer': self.manufacturer,
+			'Product': self.product
+		}
+
 	@classmethod
 	def parse_arg(cls, arg: Dict[str, str]) -> 'Fido2Device':
 		return Fido2Device(
@@ -916,12 +972,13 @@ class LsblkInfo:
 	name: str = ''
 	path: Path = Path()
 	pkname: str = ''
-	size: Size = field(default_factory=lambda: Size(0, Unit.B))
+	size: Size = field(default_factory=lambda: Size(0, Unit.B, SectorSize.default()))
 	log_sec: int = 0
 	pttype: str = ''
 	ptuuid: str = ''
 	rota: bool = False
 	tran: Optional[str] = None
+	partn: Optional[int] = None
 	partuuid: Optional[str] = None
 	parttype :Optional[str] = None
 	uuid: Optional[str] = None
@@ -946,6 +1003,7 @@ class LsblkInfo:
 			'ptuuid': self.ptuuid,
 			'rota': self.rota,
 			'tran': self.tran,
+			'partn': self.partn,
 			'partuuid': self.partuuid,
 			'parttype' : self.parttype,
 			'uuid': self.uuid,
@@ -993,7 +1051,8 @@ class LsblkInfo:
 			if isinstance(getattr(lsblk_info, data_field), Path):
 				val = Path(blockdevice[lsblk_field])
 			elif isinstance(getattr(lsblk_info, data_field), Size):
-				val = Size(blockdevice[lsblk_field], Unit.B)
+				sector_size = SectorSize(blockdevice['log-sec'], Unit.B)
+				val = Size(blockdevice[lsblk_field], Unit.B, sector_size)
 			else:
 				val = blockdevice[lsblk_field]
 
@@ -1080,7 +1139,6 @@ def get_lsblk_info(dev_path: Union[Path, str]) -> LsblkInfo:
 def get_all_lsblk_info() -> List[LsblkInfo]:
 	return _fetch_lsblk_info()
 
-
 def get_lsblk_by_mountpoint(mountpoint: Path, as_prefix: bool = False) -> List[LsblkInfo]:
 	def _check(infos: List[LsblkInfo]) -> List[LsblkInfo]:
 		devices = []
