@@ -3,6 +3,7 @@ from typing import Optional, TYPE_CHECKING
 if TYPE_CHECKING:
 	from .general import SysCommandWorker
 
+
 class RequirementError(BaseException):
 	pass
 
@@ -15,10 +16,6 @@ class UnknownFilesystemFormat(BaseException):
 	pass
 
 
-class ProfileError(BaseException):
-	pass
-
-
 class SysCallError(BaseException):
 	def __init__(self, message :str, exit_code :Optional[int] = None, worker :Optional['SysCommandWorker'] = None) -> None:
 		super(SysCallError, self).__init__(message)
@@ -27,22 +24,10 @@ class SysCallError(BaseException):
 		self.worker = worker
 
 
-class PermissionError(BaseException):
-	pass
-
-
-class ProfileNotFound(BaseException):
-	pass
-
-
 class HardwareIncompatibilityError(BaseException):
 	pass
 
 
-class UserError(BaseException):
-	pass
-
-
 class ServiceException(BaseException):
 	pass
 
@@ -51,9 +36,5 @@ class PackageError(BaseException):
 	pass
 
 
-class TranslationError(BaseException):
-	pass
-
-
 class Deprecated(BaseException):
-	pass
\ No newline at end of file
+	pass
