@@ -30,7 +30,7 @@ if TYPE_CHECKING:
 
 
 def generate_password(length :int = 64) -> str:
-	haystack = string.printable # digits, ascii_letters, punctiation (!"#$[] etc) and whitespace
+	haystack = string.printable # digits, ascii_letters, punctuation (!"#$[] etc) and whitespace
 	return ''.join(secrets.choice(haystack) for i in range(length))
 
 
@@ -69,8 +69,6 @@ def jsonify(obj: Any, safe: bool = True) -> Any:
 		# a dictionary representation of the object so that it can be
 		# processed by the json library.
 		return jsonify(obj.json(), safe)
-	if hasattr(obj, '__dump__'):
-		return obj.__dump__()
 	if isinstance(obj, (datetime, date)):
 		return obj.isoformat()
 	if isinstance(obj, (list, set, tuple)):
@@ -217,6 +215,7 @@ class SysCommandWorker:
 
 		if self.child_fd:
 			return os.write(self.child_fd, data + (b'\n' if line_ending else b''))
+			os.fsync(self.child_fd)
 
 		return 0
 
@@ -460,12 +459,14 @@ def _pid_exists(pid: int) -> bool:
 def run_custom_user_commands(commands :List[str], installation :Installer) -> None:
 	for index, command in enumerate(commands):
 		script_path = f"/var/tmp/user-command.{index}.sh"
-		chroot_path = installation.target / script_path
-		
+		chroot_path = f"{installation.target}/{script_path}"
+
 		info(f'Executing custom command "{command}" ...')
-		chroot_path.write_text(command)
+		with open(chroot_path, "w") as user_script:
+			user_script.write(command)
+
 		SysCommand(f"arch-chroot {installation.target} bash {script_path}")
-		
+
 		os.unlink(chroot_path)
 
 
